@@ -206,7 +206,7 @@ class TH(object):
         for value in values:
             raise NotImplementedError
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_tobject(cursor) +
-            cursor.return_string(b"") + cursor.return_fields(self._format_tlist, len(values)))
+            cursor.return_string(b"") + cursor.return_fields(self._format_tlist, len(values))) #FIXME
     def length_tlist(self, values):
         return self.length_tobject() + uproot.write.sink.cursor.Cursor.length_string(b"") + self._format_tlist.size + sum(0 for x in values) + self._format_cntvers.size
 
@@ -270,7 +270,7 @@ class TH(object):
     def return_taxis(self, cursor, axis):
         cnt = numpy.int64(self.length_taxis(axis) - 4) | uproot.const.kByteCountMask
         vers = 10
-        if axis["_fFirst"] != 0 or axis["_fLast"] != 0 or axis["_fBits2"] != 0 or axis["_fTimeDisplay"] or axis["_fTimeFormat"] != b"" or axis["_fLabels"] or axis["_fModLabs"]:
+        if len(axis.get("_fLabels", [])) > 0 or len(axis.get("_fModLabs", [])) > 0:
             raise NotImplementedError
         return (cursor.return_fields(self._format_cntvers, cnt, vers) +
                 self.return_tnamed(cursor, axis["_fName"], axis["_fTitle"]) +
@@ -280,13 +280,13 @@ class TH(object):
                             axis["_fXmin"],
                             axis["_fXmax"]) +
                 self.return_tarray(cursor, axis["_fXbins"]) +
-                (b"\x00" * 20))
-        # cursor.write_fields(sink, self._format_taxis_2,
-        #                     axis["_fFirst"],
-        #                     axis["_fLast"],
-        #                     axis["_fBits2"],
-        #                     axis["_fTimeDisplay"])
-        # cursor.write_string(sink, axis["_fTimeFormat"])
+                cursor.return_fields(self._format_taxis_2,
+                                    axis["_fFirst"],
+                                    axis["_fLast"],
+                                    axis["_fBits2"],
+                                    axis["_fTimeDisplay"]) +
+                cursor.return_string(axis["_fTimeFormat"]) +
+                (b"\x00" * 8)) # duck typing
         # self.write_tlist(cursor, sink, axis["_fLabels"])
         # self.write_tlist(cursor, sink, axis["_fModLabs"])
     def length_taxis(self, axis):
@@ -294,7 +294,10 @@ class TH(object):
                 self.length_tattaxis() +
                 self._format_taxis_1.size +
                 self.length_tarray(axis["_fXbins"]) +
-                20 + 6)
+                self._format_taxis_2.size +
+                uproot.write.sink.cursor.Cursor.length_string(axis["_fTimeFormat"]) +
+                8 + # Duck typing
+                self._format_cntvers.size)
                 # self._format_taxis_2.size +
                 # uproot.write.sink.cursor.Cursor.length_string(axis["_fTimeFormat"]) +
                 # self.length_tlist(axis["_fLabels"]) +
@@ -341,7 +344,7 @@ class TH(object):
     _format_th2_1 = struct.Struct(">dddd")
     def return_th2(self, cursor, name):
         cnt = numpy.int64(self.length_th2(name) - 4) | uproot.const.kByteCountMask
-        vers = 4
+        vers = 5
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th1(cursor, name) +
                 cursor.return_fields(self._format_th2_1,
                                      self.fields["_fScalefactor"],
@@ -352,7 +355,7 @@ class TH(object):
     _format_th3_1 = struct.Struct(">ddddddd")
     def return_th3(self, cursor, name):
         cnt = numpy.int64(self.length_th3(name) - 4) | uproot.const.kByteCountMask
-        vers = 5
+        vers = 6
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th1(cursor, name) +
                 self.return_tatt3d(cursor) + cursor.return_fields(self._format_th3_1,
                                                                   self.fields["_fTsumwy"],
@@ -395,7 +398,7 @@ class TH(object):
 
     def return_th1d(self, cursor, name):
         cnt = numpy.int64(self.length_th1d(name) - 4) | uproot.const.kByteCountMask
-        vers = 2
+        vers = 3
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th1(cursor, name)
                 + self.return_tarray(cursor, self.valuesarray))
 
@@ -404,7 +407,7 @@ class TH(object):
 
     def return_th2d(self, cursor, name):
         cnt = numpy.int64(self.length_th2d(name) - 4) | uproot.const.kByteCountMask
-        vers = 3
+        vers = 4
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th2(cursor, name)
                 + self.return_tarray(cursor, self.valuesarray))
 
@@ -413,7 +416,7 @@ class TH(object):
 
     def return_th3d(self, cursor, name):
         cnt = numpy.int64(self.length_th3d(name) - 4) | uproot.const.kByteCountMask
-        vers = 3
+        vers = 4
         return (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th3(cursor, name)
                 + self.return_tarray(cursor, self.valuesarray))
 
@@ -425,33 +428,33 @@ class TH(object):
         givenbytes = 0
         cnt = numpy.int64(self.length(name) - 4) | uproot.const.kByteCountMask
         if "TH1" in self.fClassName.decode("utf-8"):
-            vers = 2
+            vers = 3
             givenbytes = cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th1(cursor, name)
             givenbytes += self.return_tarray(cursor, self.valuesarray)
         elif "TH2" in self.fClassName.decode("utf-8"):
-            vers = 3
+            vers = 4
             givenbytes = cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th2(cursor, name)
             givenbytes += self.return_tarray(cursor, self.valuesarray)
         elif "TH3" in self.fClassName.decode("utf-8"):
-            vers = 3
+            vers = 4
             givenbytes = cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th3(cursor, name)
             givenbytes += self.return_tarray(cursor, self.valuesarray)
         elif "TProfile" == self.fClassName.decode("utf-8"):
-            vers = 6
+            vers = 7
             givenbytes = (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th1d(cursor, name)
                             + self.return_tarray(cursor, self.fields["_fBinEntries"]) +
                             cursor.return_fields(self._format_tprofile, self.fields["_fErrorMode"], self.fields["_fYmin"],
                             self.fields["_fYmax"], self.fields["_fTsumwy"], self.fields["_fTsumwy2"]) +
                             self.return_tarray(cursor, self.fields["_fBinSumw2"]))
         elif "TProfile2D" == self.fClassName.decode("utf-8"):
-            vers = 7
+            vers = 8
             givenbytes = (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th2d(cursor, name)
                             + self.return_tarray(cursor, self.fields["_fBinEntries"]) +
                             cursor.return_fields(self._format_tprofile, self.fields["_fErrorMode"], self.fields["_fZmin"],
                             self.fields["_fZmax"], self.fields["_fTsumwz"], self.fields["_fTsumwz2"]) +
                             self.return_tarray(cursor, self.fields["_fBinSumw2"]))
         elif "TProfile3D" == self.fClassName.decode("utf-8"):
-            vers = 7
+            vers = 8
             givenbytes = (cursor.return_fields(self._format_cntvers, cnt, vers) + self.return_th3d(cursor, name)
                             + self.return_tarray(cursor, self.fields["_fBinEntries"]) +
                             cursor.return_fields(self._format_tprofile, self.fields["_fErrorMode"], self.fields["_fTmin"],
